from flask import Flask, request, redirect, url_for, render_template
import sqlite3

app = Flask(__name__)

# Initialize the database
def init_db():
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('''CREATE TABLE IF NOT EXISTS proposals
                 (id INTEGER PRIMARY KEY, title TEXT, body TEXT)''')
    c.execute('''CREATE TABLE IF NOT EXISTS votes
                 (id INTEGER PRIMARY KEY, proposal_id INTEGER, vote_type TEXT, reason TEXT,
                  FOREIGN KEY(proposal_id) REFERENCES proposals(id))''')
    c.execute('''CREATE TABLE IF NOT EXISTS arguments
                 (id INTEGER PRIMARY KEY, proposal_id INTEGER, argument TEXT,
                  FOREIGN KEY(proposal_id) REFERENCES proposals(id))''')
    conn.commit()
    conn.close()

init_db()

@app.route('/')
def index():
    proposals = get_proposals_with_votes()
    return render_template('index.html', proposals=proposals)

@app.route('/submit', methods=['GET', 'POST'])
def submit():
    if request.method == 'POST':
        title = request.form['title']
        body = request.form['body']
        conn = sqlite3.connect('database.db')
        c = conn.cursor()
        c.execute('INSERT INTO proposals (title, body) VALUES (?, ?)', (title, body))
        conn.commit()
        conn.close()
        return redirect(url_for('index'))
    return render_template('submit.html')

@app.route('/proposal/<int:proposal_id>', methods=['GET', 'POST'])
def proposal(proposal_id):
    if request.method == 'POST':
        vote_type = request.form.get('vote_type')
        reason = request.form.get('reason')
        decrement_vote_type = request.form.get('decrement_vote_type')
        argument = request.form.get('argument')
        if vote_type:
            submit_vote(proposal_id, vote_type, reason)
        elif decrement_vote_type:
            decrement_vote(proposal_id, decrement_vote_type)
        elif argument:
            submit_argument(proposal_id, argument)
        return redirect(url_for('proposal', proposal_id=proposal_id))
    proposal = get_proposal(proposal_id)
    vote_counts = get_vote_counts(proposal_id)
    arguments = get_arguments(proposal_id)
    return render_template('proposal.html', proposal=proposal, vote_counts=vote_counts, arguments=arguments)

@app.route('/clear_votes', methods=['POST'])
def clear_votes():
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('DELETE FROM votes')
    c.execute('DELETE FROM arguments')
    conn.commit()
    conn.close()
    return redirect(url_for('index'))

@app.route('/delete_proposal/<int:proposal_id>', methods=['POST'])
def delete_proposal(proposal_id):
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('DELETE FROM votes WHERE proposal_id = ?', (proposal_id,))
    c.execute('DELETE FROM proposals WHERE id = ?', (proposal_id,))
    conn.commit()
    conn.close()
    return redirect(url_for('index'))

@app.route('/decrement_vote/<int:proposal_id>', methods=['POST'])
def decrement_vote_route(proposal_id):
    vote_type = request.form['decrement_vote_type']
    decrement_vote(proposal_id, vote_type)
    return redirect(url_for('proposal', proposal_id=proposal_id))

@app.route('/view_database')
def view_database():
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('SELECT * FROM proposals')
    proposals = c.fetchall()
    c.execute('SELECT proposal_id, vote_type, reason FROM votes')
    votes = c.fetchall()
    c.execute('SELECT proposal_id, argument FROM arguments')
    arguments = c.fetchall()
    conn.close()
    
    proposal_data = {}
    for proposal in proposals:
        proposal_id, title, body = proposal
        proposal_data[proposal_id] = {
            'title': title,
            'body': body,
            'votes': {
                'block': [],
                'stand_aside': [],
                'reservations': [],
                'agree': []
            },
            'arguments': []
        }
    
    for vote in votes:
        proposal_id, vote_type, reason = vote
        if proposal_id in proposal_data:
            proposal_data[proposal_id]['votes'][vote_type].append(reason)
    
    for argument in arguments:
        proposal_id, argument_text = argument
        if proposal_id in proposal_data:
            proposal_data[proposal_id]['arguments'].append(argument_text)
    
    return render_template('view_database.html', proposal_data=proposal_data)

@app.route('/reword_proposal/<int:proposal_id>', methods=['POST'])
def reword_proposal(proposal_id):
    new_text = request.form['new_text']
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('UPDATE proposals SET body = ? WHERE id = ?', (new_text, proposal_id))
    conn.commit()
    conn.close()
    return redirect(url_for('proposal', proposal_id=proposal_id))

@app.route('/submit_argument/<int:proposal_id>', methods=['POST'])
def submit_argument(proposal_id):
    argument = request.form['argument']
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('INSERT INTO arguments (proposal_id, argument) VALUES (?, ?)', (proposal_id, argument))
    conn.commit()
    conn.close()
    return redirect(url_for('proposal', proposal_id=proposal_id))

def get_proposals_with_votes():
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('''
        SELECT p.id, p.title, 
               COALESCE(SUM(CASE WHEN v.vote_type = 'block' THEN 1 ELSE 0 END), 0) AS block,
               COALESCE(SUM(CASE WHEN v.vote_type = 'stand_aside' THEN 1 ELSE 0 END), 0) AS stand_aside,
               COALESCE(SUM(CASE WHEN v.vote_type = 'reservations' THEN 1 ELSE 0 END), 0) AS reservations,
               COALESCE(SUM(CASE WHEN v.vote_type = 'agree' THEN 1 ELSE 0 END), 0) AS agree,
               COUNT(v.id) AS total_votes
        FROM proposals p
        LEFT JOIN votes v ON p.id = v.proposal_id
        GROUP BY p.id, p.title
    ''')
    proposals = c.fetchall()
    conn.close()
    return [(id, title, {'block': block, 'stand_aside': stand_aside, 'reservations': reservations, 'agree': agree}, total_votes) for id, title, block, stand_aside, reservations, agree, total_votes in proposals]

def submit_vote(proposal_id, vote_type, reason):
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('SELECT COUNT(*) FROM votes WHERE proposal_id = ?', (proposal_id,))
    vote_count = c.fetchone()[0]
    if vote_count < 10:
        c.execute('INSERT INTO votes (proposal_id, vote_type, reason) VALUES (?, ?, ?)', (proposal_id, vote_type, reason))
        conn.commit()
    conn.close()

def decrement_vote(proposal_id, vote_type):
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('SELECT id FROM votes WHERE proposal_id = ? AND vote_type = ? LIMIT 1', (proposal_id, vote_type))
    vote = c.fetchone()
    if vote:
        c.execute('DELETE FROM votes WHERE id = ?', (vote[0],))
        conn.commit()
    conn.close()

def get_proposal(proposal_id):
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('SELECT id, title, body FROM proposals WHERE id = ?', (proposal_id,))
    proposal = c.fetchone()
    conn.close()
    return proposal

def get_vote_counts(proposal_id):
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('SELECT vote_type, COUNT(*) FROM votes WHERE proposal_id = ? GROUP BY vote_type', (proposal_id,))
    votes = c.fetchall()
    vote_counts = {vote_type: count for vote_type, count in votes}
    conn.close()
    return vote_counts

def get_arguments(proposal_id):
    conn = sqlite3.connect('database.db')
    c = conn.cursor()
    c.execute('SELECT argument FROM arguments WHERE proposal_id = ? ORDER BY id DESC', (proposal_id,))
    arguments = [row[0] for row in c.fetchall()]
    conn.close()
    return arguments

if __name__ == '__main__':
    app.run(debug=True, host='127.0.0.1', port=5000)